import 'package:app_popup_menu/app_popup_menu.dart';
import 'package:flutter/material.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:flutter/src/widgets/placeholder.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_progress_hud/flutter_progress_hud.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:fluttericon/font_awesome_icons.dart';
import 'package:intl/intl.dart';
import 'package:unit2/bloc/profile/profile_bloc.dart';
import 'package:unit2/bloc/user/user_bloc.dart';
import 'package:unit2/model/profile/learning_development.dart';
import 'package:unit2/screens/profile/components/learning_development/edit_modal.dart';
import 'package:unit2/theme-data.dart/box_shadow.dart';
import 'package:unit2/theme-data.dart/colors.dart';
import 'package:unit2/utils/global.dart';
import 'package:unit2/utils/text_container.dart';
import 'package:unit2/widgets/Leadings/add_leading.dart';
import 'package:unit2/widgets/empty_data.dart';
import 'package:unit2/widgets/error_state.dart';
import '../../../bloc/profile/learningDevelopment/learning_development_bloc.dart';
import '../../../utils/alerts.dart';
import '../../../widgets/Leadings/close_leading.dart';
import 'learning_development/add_modal.dart';

class LearningAndDevelopmentScreen extends StatelessWidget {
  const LearningAndDevelopmentScreen({
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    String token;
    int profileId;
    DateFormat dteFormat2 = DateFormat.yMMMMd('en_US');
    return Scaffold(
        appBar: AppBar(
          title: context.watch<LearningDevelopmentBloc>().state is LearningDevelopmentAddingState? const FittedBox(child: Text("Add $learningAndDevelopmentScreenTitle")):context.watch<LearningDevelopmentBloc>().state is LearningDevelopmentUpdatingState?const FittedBox(child: Text("Edit $learningAndDevelopmentScreenTitle")):const FittedBox(child: Text(learningAndDevelopmentScreenTitle)),
          centerTitle: true,
          backgroundColor: primary,
          actions: (context.watch<LearningDevelopmentBloc>().state is LearningDevelopmentLoadedState)?[
            AddLeading(onPressed: () {
              context.read<LearningDevelopmentBloc>().add(ShowAddLearningDevelopmentForm());
            })
          ]:(context.watch<LearningDevelopmentBloc>().state is LearningDevelopmentAddingState || context.watch<LearningDevelopmentBloc>().state is LearningDevelopmentUpdatingState)?[
            CloseLeading(onPressed:() {
              context.read<LearningDevelopmentBloc>().add(LoadLearniningDevelopment());
            })
          ]:[]
        ),
        body: ProgressHUD(
          padding: const EdgeInsets.all(24),
          indicatorWidget: const SpinKitFadingCircle(
            color: Colors.white,
          ),
          backgroundColor: Colors.black87,
          child: BlocBuilder<UserBloc, UserState>(
            builder: (context, state) {
              if (state is UserLoggedIn) {
                token = state.userData!.user!.login!.token!;
                profileId = state.userData!.user!.login!.user!.profileId!;
                return BlocBuilder<ProfileBloc, ProfileState>(
                  builder: (context, state) {
                    if (state is ProfileLoaded) {
                      return BlocConsumer<LearningDevelopmentBloc,
                          LearningDevelopmentState>(
                        listener: (context, state) {
                          if (state is LearningDevelopmentLoadingState) {
                            final progress = ProgressHUD.of(context);
                            progress!.showWithText("Please wait...");
                          }
                          if (state is LearningDevelopmentLoadedState ||
                              state is LearningDevelopmentErrorState ||
                              state is LearningDevelopmentAddingState ||
                              state is LearningDevelopmentAddedState ||
                              state is LearningDevelopmentUpdatingState) {
                            final progress = ProgressHUD.of(context);
                            progress!.dismiss();
                          }
                          //// Added State
                          if (state is LearningDevelopmentAddedState) {
                            if (state.response['success']) {
                              successAlert(context, "Adding Successfull!",
                                  state.response['message'], () {
                                Navigator.of(context).pop();
                                context
                                    .read<LearningDevelopmentBloc>()
                                    .add(LoadLearniningDevelopment());
                              });
                            } else {
                              errorAlert(context, "Adding Failed",
                                  "Something went wrong. Please try again.",
                                  () {
                                Navigator.of(context).pop();
                                context
                                    .read<LearningDevelopmentBloc>()
                                    .add(LoadLearniningDevelopment());
                              });
                            }
                          }
                          ////Updated State
                                    if (state is LearningDevelopmentUpdatedState) {
                            if (state.response['success']) {
                              successAlert(context, "Update Successfull!",
                                  state.response['message'], () {
                                Navigator.of(context).pop();
                                context
                                    .read<LearningDevelopmentBloc>()
                                    .add(LoadLearniningDevelopment());
                              });
                            } else {
                              errorAlert(context, "Update Failed",
                                  "Something went wrong. Please try again.",
                                  () {
                                Navigator.of(context).pop();
                                context
                                    .read<LearningDevelopmentBloc>()
                                    .add(LoadLearniningDevelopment());
                              });
                            }
                          }
                          ////Deleted State
                          if (state is DeleteLearningDevelopmentState) {
                            if (state.success) {
                              successAlert(context, "Deletion Successfull!",
                                  "Learning Development Has Been Deleted Successfully", () {
                                Navigator.of(context).pop();
                                context
                                    .read<LearningDevelopmentBloc>()
                                    .add(LoadLearniningDevelopment());
                              });
                            } else {
                              errorAlert(context, "Deletion Failed",
                                  "Something went wrong. Please try again.",
                                  () {
                                Navigator.of(context).pop();
                                context
                                    .read<LearningDevelopmentBloc>()
                                    .add(LoadLearniningDevelopment());
                              });
                            }
                          }
                          // TODO: implement listener
                        },
                        builder: (context, state) {
                          if (state is LearningDevelopmentLoadedState) {
                            if (state.learningsAndDevelopment.isNotEmpty) {
                              return ListView.builder(
                                  padding: const EdgeInsets.symmetric(
                                      vertical: 8, horizontal: 10),
                                  itemCount:
                                      state.learningsAndDevelopment.length,
                                  itemBuilder:
                                      (BuildContext context, int index) {
                                    String training = state
                                        .learningsAndDevelopment[index]
                                        .conductedTraining!
                                        .title!
                                        .title!;
                                    String provider = state
                                        .learningsAndDevelopment[index]
                                        .conductedTraining!
                                        .conductedBy!
                                        .name!;
                                    String start = dteFormat2.format(state
                                        .learningsAndDevelopment[index]
                                        .conductedTraining!
                                        .fromDate!);
                                    String end = dteFormat2.format(state
                                        .learningsAndDevelopment[index]
                                        .conductedTraining!
                                        .toDate!);
                                    String type = state
                                        .learningsAndDevelopment[index]
                                        .conductedTraining!
                                        .learningDevelopmentType!
                                        .title!;
                                    return Column(
                                      children: [
                                        Container(
                                          decoration: box1(),
                                          padding: const EdgeInsets.symmetric(
                                              horizontal: 12, vertical: 8),
                                          width: screenWidth,
                                          child: Row(
                                            children: [
                                              Expanded(
                                                child: Column(
                                                    mainAxisAlignment:
                                                        MainAxisAlignment.start,
                                                    crossAxisAlignment:
                                                        CrossAxisAlignment
                                                            .start,
                                                    children: [
                                                      Text(
                                                        training,
                                                        style: Theme.of(context)
                                                            .textTheme
                                                            .titleMedium!
                                                            .copyWith(
                                                                fontWeight:
                                                                    FontWeight
                                                                        .w600,color: primary),
                                                      ),
                                               
                                                      const SizedBox(
                                                        height: 8,
                                                      ),
                                                      Text(
                                                        provider,
                                                        style: Theme.of(context)
                                                            .textTheme
                                                            .titleSmall,
                                                      ),
                                                      const SizedBox(
                                                        height: 5,
                                                      ),
                                                      Text(
                                                        "$duration: $start to $end",
                                                        style: Theme.of(context)
                                                            .textTheme
                                                            .labelMedium,
                                                      ),
                                                      const SizedBox(
                                                        height: 5,
                                                      ),
                                                    ]),
                                              ),
                                              AppPopupMenu<int>(
                                                offset: const Offset(-10, -10),
                                                elevation: 3,
                                                onSelected: (value) {
                                                  ////delete -= = = = = = = = =>>
                                                  if (value == 2) {
                                                    confirmAlert(context, () {
                                                      final progress =
                                                          ProgressHUD.of(
                                                              context);
                                                      progress!.showWithText(
                                                          "Loading...");
                                                      BlocProvider.of<
                                                                  LearningDevelopmentBloc>(
                                                              context)
                                                          .add(DeleteLearningDevelopment(
                                                              trainingId: state
                                                                  .learningsAndDevelopment[
                                                                      index]
                                                                  .conductedTraining!
                                                                  .id!,
                                                              hours: state
                                                                  .learningsAndDevelopment[
                                                                      index]
                                                                  .conductedTraining!
                                                                  .totalHours!,
                                                              sponsorId: state
                                                                  .learningsAndDevelopment[
                                                                      index]
                                                                  .sponsoredBy
                                                                  ?.id,
                                                              profileId:
                                                                  profileId,
                                                              token: token));
                                                    }, "Delete?",
                                                        "Confirm Delete?");
                                                  }
                                                  if (value == 1) {
                                                    bool isOverseas;
                                                    ////edit  = = = = = = = =>>
                                                    final progress =
                                                        ProgressHUD.of(context);
                                                    progress!.showWithText(
                                                        "Loading...");

                                                    if (state
                                                            .learningsAndDevelopment[
                                                                index]
                                                            .conductedTraining
                                                            ?.venue
                                                            ?.cityMunicipality ==
                                                        null) {
                                                      isOverseas = true;
                                                    } else {
                                                      isOverseas = false;
                                                    }
                                                    context
                                                        .read<
                                                            LearningDevelopmentBloc>()
                                                        .add(ShowEditLearningDevelopmentForm(
                                                            profileId:
                                                                profileId,
                                                            token: token,
                                                            learningDevelopment:
                                                                state.learningsAndDevelopment[
                                                                    index],
                                                            isOverseas:
                                                                isOverseas));
                                                  }
                                                },
                                              menuItems: [
                                                  popMenuItem(
                                                      text: "Update",
                                                      value: 1,
                                                      icon: Icons.edit),
                                                  popMenuItem(
                                                      text: "Remove",
                                                      value: 2,
                                                      icon: Icons.delete),
                                                           popMenuItem(
                                                      text: "Attach",
                                                      value: 2,
                                                      icon: Icons.attach_file),
                                              
                                                ],
                                                icon: const Icon(
                                                  Icons.more_vert,
                                                  color: Colors.grey,
                                                ),
                                                tooltip: "Options",
                                              )
                                            ],
                                          ),
                                        ),
                                        const SizedBox(
                                          height: 8,
                                        ),
                                      ],
                                    );
                                  });
                            } else {
                              return const EmptyData(
                                  message:
                                      "You don't have any Learning and Development added. Please click + to add.");
                            }
                          }
                          if (state is LearningDevelopmentErrorState) {
                            return (SomethingWentWrong(
                                message: state.message, onpressed: () {context.read<LearningDevelopmentBloc>().add(GetLearningDevelopments(profileId: profileId, token: token));}));
                          }
                          if (state is LearningDevelopmentAddingState) {
                            return AddLearningAndDevelopmentScreen(
                              token: token,
                              profileId: profileId,
                            );
                          }
                          if (state is LearningDevelopmentUpdatingState) {
                            return EditLearningAndDevelopmentScreen(
                              token: token,
                              profileId: profileId,
                            );
                          }
                          return Container();
                        },
                      );
                    }
                    return Container();
                  },
                );
              }
              return Container();
            },
          ),
        ));
  }

  PopupMenuItem<int> popMenuItem({String? text, int? value, IconData? icon}) {
    return PopupMenuItem(
      value: value,
      child: Row(
        children: [
          Icon(
            icon,
          ),
          const SizedBox(
            width: 10,
          ),
          Text(
            text!,
          ),
        ],
      ),
    );
  }
}
